@@ -3,6 +3,7 @@ import unittest
 import doctest
 import inspect
 import textwrap
+import base64
 
 OPTIONFLAGS = (doctest.ELLIPSIS |
                doctest.NORMALIZE_WHITESPACE)
@@ -26,15 +27,25 @@ def verify_source(source):
     tree = fix_tree(parse(source, 'exec'))
     code = pycodegen.ModuleCodeGenerator(tree).getCode()
     generator = ModuleSourceCodeGenerator(tree)
-    source = generator.getSourceCode()
-    tree = fix_tree(parse(source, 'exec'))
-    if code.co_code != pycodegen.ModuleCodeGenerator(tree).getCode().co_code:
-        return source
+    generated = generator.getSourceCode()
+    new = fix_tree(parse(generated, 'exec'))
+
+    old = code.co_code
+    new = pycodegen.ModuleCodeGenerator(new).getCode().co_code
+
+    if old != new:
+        return generated
 
 def verify(func):
-    return lambda suite: suite.assertEqual(verify_source(
-        textwrap.dedent("\n".join(
-        inspect.getsource(func).split('\n')[2:]))), None)
+    def test(suite):
+        source = textwrap.dedent(
+            "\n".join(inspect.getsource(func).split('\n')[2:])
+            )
+
+        result = verify_source(source)
+        return suite.assertEqual(result, None, "%r != %r" % (source, result))
+
+    return test
 
 class TestSourceCodeGeneration(unittest.TestCase):
     """The ``verify`` decorator is used to create a test-case out of
@@ -121,19 +132,25 @@ class TestSourceCodeGeneration(unittest.TestCase):
         del foo.bar
         del foo[bar]
         del foo[bar:boo]
-        
+
     @verify
     def testListComprehensions(self):
         [x for x in xs]
         [x for x in xs if x]
         [x for x in xs if x == y]
-        [x*y for x in xs for y in ys]
-        
+
+    @verify
+    def testMultipleListComprehensions(self):
+        [(x * y) for x in xs for y in ys]
+
     @verify
     def testGeneratorComprehensions(self):
         (x for x in xs)
         (x for x in xs if x)
         (x for x in xs if x == y)
+
+    @verify
+    def testMultipleGeneratorComprehensions(self):
         (x*y for x in xs for x in xs for y in ys)
 
     @verify
@@ -358,3 +375,11 @@ class TestSourceCodeGeneration(unittest.TestCase):
     def testContinue(self):
         for i in range(5):
             continue
+
+    @verify
+    def testFunctionDocString(self):
+        def foo():
+            """bar."""
+
+        def bar():
+            "hello"
